@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, Ghostgum Software Pty Ltd.  All rights reserved.
+/* Copyright (C) 2001-2004, Ghostgum Software Pty Ltd.  All rights reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
@@ -14,11 +14,11 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861.
  */
 
+/* $Id: gdevdsp2.h,v 1.8 2004/07/07 09:33:19 ghostgum Exp $ */
 /* gdevdsp2.c */
-/* $Id: gdevdsp2.h,v 1.2.2.2 2002/04/10 09:33:25 giles Exp $ */
 
-#ifndef gx_device_display_DEFINED
-#  define gx_device_display_DEFINED
+#ifndef gdevdsp2_INCLUDED
+#  define gdevdsp2_INCLUDED
 
 typedef struct gx_device_display_s gx_device_display;
 
@@ -28,7 +28,10 @@ typedef struct gx_device_display_s gx_device_display;
 	void *pHandle;\
 	int nFormat;\
 	void *pBitmap;\
-	unsigned long ulBitmapSize
+	unsigned long ulBitmapSize;\
+	int HWResolution_set;\
+        gs_devn_params devn_params;\
+        equivalent_cmyk_color_params equiv_cmyk_colors
 
 /* The device descriptor */
 struct gx_device_display_s {
@@ -42,6 +45,4 @@ extern_st(st_device_display);
     "gx_device_display", display_enum_ptrs, display_reloc_ptrs,\
     gx_device_finalize)
 
-
-#endif
-
+#endif /* gdevdsp2_INCLUDED */
