@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevpdfj.c,v 1.43 2005/04/04 20:28:27 igor Exp $ */
+/* $Id: gdevpdfj.c,v 1.49 2005/09/06 13:47:10 leonardo Exp $ */
 /* Image-writing utilities for pdfwrite driver */
 #include "memory_.h"
 #include "string_.h"
@@ -26,6 +26,7 @@
 #include "gxcspace.h"
 #include "gsiparm4.h"
 #include "gdevpsds.h"
+#include "spngpx.h"
 
 #define CHECK(expr)\
   BEGIN if ((code = (expr)) < 0) return code; END
@@ -109,11 +110,12 @@ pdf_put_pixel_image_values(cos_dict_t *pcd, gx_device_pdf *pdev,
     {
 	int i;
 
-	for (i = 0; i < num_components * 2; ++i)
+	for (i = 0; i < num_components * 2; ++i) {
 	    if (pim->Decode[i] !=
 		(default_decode ? default_decode[i] : i & 1)
 		)
 		break;
+	}
 	if (i < num_components * 2) {
 	    cos_array_t *pca =
 		cos_array_alloc(pdev, "pdf_put_pixel_image_values(decode)");
@@ -372,10 +374,10 @@ pdf_make_alt_stream(gx_device_pdf * pdev, psdf_binary_writer * pbw)
         return_error(gs_error_VMerror);
     pbw->dev = (gx_device_psdf *)pdev;
     pbw->memory = pdev->pdf_memory;
-    pbw->target = NULL; /* We don't need target with cos_write_stream. */
     pdev->strm = pbw->strm;
     code = psdf_begin_binary((gx_device_psdf *) pdev, pbw);
     pdev->strm = save_strm;
+    pbw->target = NULL; /* We don't need target with cos_write_stream. */
     return code;
 }
 
@@ -406,8 +408,9 @@ pdf_complete_image_data(gx_device_pdf *pdev, pdf_image_writer *piw, int data_h,
 			int width, int bits_per_pixel)
 {
     if (data_h != piw->height) {
-	if (piw->binary[0].strm->procs.process == s_DCTE_template.process) {
-	    /* 	Since DCTE can't safely close with incomplete data,
+	if (piw->binary[0].strm->procs.process == s_DCTE_template.process || 
+	    piw->binary[0].strm->procs.process == s_PNGPE_template.process ) {
+	    /* 	Since DCTE and PNGPE can't safely close with incomplete data,
 		we add stub data to complete the stream.
 	    */
 	    int bytes_per_line = (width * bits_per_pixel + 7) / 8;
@@ -447,12 +450,6 @@ pdf_end_image_binary(gx_device_pdf *pdev, pdf_image_writer *piw, int data_h)
     return code < 0 ? code : code1;
 }
 
-private int 
-nocheck(gx_device_pdf * pdev, pdf_resource_t *pres0, pdf_resource_t *pres1)
-{
-    return 1;
-}
-
 /*
  * Finish writing an image.  If in-line, write the BI/dict/ID/data/EI and
  * return 1; if a resource, write the resource definition and return 0.
@@ -469,6 +466,11 @@ pdf_end_write_image(gx_device_pdf * pdev, pdf_image_writer * piw)
 	int code;
 
 	if (named) {
+	    if (pdev->ForOPDFRead) {
+		code = cos_dict_put_c_key_bool(named, "/.Global", true);
+		if (code < 0)
+		    return code;
+	    }
 	    /*
 	     * This image was named by NI.  Copy any dictionary elements
 	     * from the named dictionary to the image stream, and then
@@ -488,21 +490,12 @@ pdf_end_write_image(gx_device_pdf * pdev, pdf_image_writer * piw)
 	    *(cos_object_t *)named = *pco;
 	    pres->object = COS_OBJECT(named);
 	} else if (!pres->named) { /* named objects are written at the end */
-	    code = pdf_find_same_resource(pdev, resourceXObject, &piw->pres, nocheck);
+	    code = pdf_substitute_resource(pdev, &piw->pres, resourceXObject, NULL, false);
 	    if (code < 0)
 		return code;
-	    if (code > 0) {
-		/*  Warning : if the image used alternate streams,
-		    its space in the pdev->streams.strm file
-		    won't be released.
-		 */
-		code = pdf_cancel_resource(pdev, pres, resourceXObject);
-		if (code < 0)
-		    return code;
-		pdf_forget_resource(pdev, pres, resourceXObject);
-		piw->pres->where_used |= pdev->used_mask;
-	    } else if (pres->object->id < 0)
-		pdf_reserve_object_id(pdev, pres, 0);
+	    /*  Warning : If the substituted image used alternate streams,
+		its space in the pdev->streams.strm file won't be released. */
+	    piw->pres->where_used |= pdev->used_mask;
 	}
 	code = pdf_add_resource(pdev, pdev->substream_Resources, "/XObject", piw->pres);
 	if (code < 0)
