@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevpdtt.c,v 1.102 2005/04/05 09:48:46 igor Exp $ */
+/* $Id: gdevpdtt.c,v 1.104 2005/10/12 08:16:50 leonardo Exp $ */
 /* Text processing for pdfwrite. */
 #include "math_.h"
 #include "string_.h"
@@ -94,8 +94,7 @@ pdf_text_set_cache(gs_text_enum_t *pte, const double *pw,
     switch (control) {
     case TEXT_SET_CHAR_WIDTH:
     case TEXT_SET_CACHE_DEVICE:
-	pdev->char_width.x = pw[0];
-	pdev->char_width.y = pw[1];
+	gs_distance_transform(pw[0], pw[1], &ctm_only(pte->pis), &pdev->char_width);
 	break;
     case TEXT_SET_CACHE_DEVICE2:
 	/*
@@ -104,8 +103,7 @@ pdf_text_set_cache(gs_text_enum_t *pte, const double *pw,
 	 * width for Widths array. Therefore we don't check 
 	 * gs_rootfont(pgs)->WMode and don't use pw[6:7].
 	 */
-	pdev->char_width.x = pw[0];
-	pdev->char_width.y = pw[1];
+	gs_distance_transform(pw[0], pw[1], &ctm_only(pte->pis), &pdev->char_width);
 	if (penum->cdevproc_callout) {
 	    memcpy(penum->cdevproc_result, pw, sizeof(penum->cdevproc_result));
 	    return 0;
@@ -203,14 +201,12 @@ pdf_text_set_cache(gs_text_enum_t *pte, const double *pw,
 			pw1, narg, control, ch, &gnstr);
 	    if (code < 0)
 		return code;
-	    if (control == TEXT_SET_CHAR_WIDTH) {
-		/* Prevent writing the clipping path to charproc.
-		   See the comment above.
-		   Note that the clipping in the graphic state will be used while 
-		   fallbacks to default implementations of graphic objects. 
-		   Hopely such fallbacks are rare. */
-		pdev->clip_path_id = gx_get_clip_path_id(penum_s->pgs);
-	    }
+	    /* Prevent writing the clipping path to charproc.
+	       See the comment above and bugs 687678, 688327.
+	       Note that the clipping in the graphic state will be used while 
+	       fallbacks to default implementations of graphic objects. 
+	       Hopely such fallbacks are rare. */
+	    pdev->clip_path_id = gx_get_clip_path_id(penum_s->pgs);
 	    penum->charproc_accum = true;
 	    return code;
 	} else {
