@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gdevpsfu.c,v 1.7.2.1 2002/02/22 19:45:56 ray Exp $ */
+/* $Id: gdevpsfu.c,v 1.12 2004/11/19 04:39:11 ray Exp $ */
 /* PostScript/PDF font writing utilities */
 #include "memory_.h"
 #include <stdlib.h>		/* for qsort */
@@ -108,26 +108,6 @@ psf_enumerate_glyphs_next(psf_glyph_enum_t *ppge, gs_glyph *pglyph)
     return ppge->enumerate_next(ppge, pglyph);
 }
 
-/*
- * Get the set of referenced glyphs (indices) for writing a subset font.
- * Does not sort or remove duplicates.
- */
-int
-psf_subset_glyphs(gs_glyph glyphs[256], gs_font *font, const byte used[32])
-{
-    int i, n;
-
-    for (i = n = 0; i < 256; ++i)
-	if (used[i >> 3] & (0x80 >> (i & 7))) {
-	    gs_glyph glyph = font->procs.encode_char(font, (gs_char)i,
-						     GLYPH_SPACE_INDEX);
-
-	    if (glyph != gs_no_glyph)
-		glyphs[n++] = glyph;
-	}
-    return n;
-}
-
 /*
  * Add composite glyph pieces to a list of glyphs.  Does not sort or
  * remove duplicates.  max_pieces is the maximum number of pieces that a
@@ -232,12 +212,13 @@ psf_check_outline_glyphs(gs_font_base *pfont, psf_glyph_enum_t *ppge,
     int code;
 
     while ((code = psf_enumerate_glyphs_next(ppge, &glyph)) != 1) {
-	gs_const_string gdata;
+	gs_glyph_data_t gdata;
 	gs_font_type1 *ignore_font;
 	gs_glyph_info_t info;
 
 	if (code < 0)
 	    return code;
+	gdata.memory = pfont->memory;
 	code = glyph_data(pfont, glyph, &gdata, &ignore_font);
 	/*
 	 * If the glyph isn't defined by a CharString, glyph_data will
@@ -250,9 +231,7 @@ psf_check_outline_glyphs(gs_font_base *pfont, psf_glyph_enum_t *ppge,
 		continue;
 	    return code;
 	}
-	if (code > 0)
-	    gs_free_const_string(pfont->memory, gdata.data, gdata.size,
-				 "psf_check_outline_glyphs");
+	gs_glyph_data_free(&gdata, "psf_check_outline_glyphs");
 	/*
 	 * If the font has a CDevProc or calls a non-standard OtherSubr,
 	 * glyph_info will return a rangecheck error.
