@@ -1,4 +1,4 @@
-/* Copyright (C) 1989, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1989-2003 artofcode LLC. All rights reserved.
   
   This software is provided AS-IS with no warranty, either express or
   implied.
@@ -14,8 +14,9 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gp_unix.c,v 1.5.2.1 2002/02/22 19:45:57 ray Exp $ */
+/* $Id: gp_unix.c,v 1.13 2004/01/15 09:27:10 giles Exp $ */
 /* Unix-specific routines for Ghostscript */
+
 #include "pipe_.h"
 #include "string_.h"
 #include "time_.h"
@@ -34,9 +35,8 @@
 #ifdef __PROTOTYPES__
 #  include <stdlib.h>		/* for exit and getenv */
 #else
-extern void exit(P1(int));
-extern char *getenv(P1(const char *));
-
+extern void exit(int);
+extern char *getenv(const char *);
 #endif
 
 /* Do platform-dependent initialization. */
@@ -55,6 +55,7 @@ gp_exit(int exit_status, int code)
 void
 gp_do_exit(int exit_status)
 {
+    exit(exit_status);
 }
 
 /* ------ Miscellaneous ------ */
@@ -68,6 +69,16 @@ gp_strerror(int errnum)
     return NULL;
 }
 
+/* read in a MacOS 'resource' from an extended attribute. */
+/* we don't try to implemented this since it requires support */
+/* for Apple's HFS(+) filesystem */
+int
+gp_read_macresource(byte *buf, const char *filename, 
+                    const uint type, const ushort id)
+{
+    return 0;
+}
+
 /* ------ Date and time ------ */
 
 /* Read the current time (in seconds since Jan. 1, 1970) */
@@ -156,3 +167,24 @@ gp_close_printer(FILE * pfile, const char *fname)
     else
 	fclose(pfile);
 }
+
+/* ------ Font enumeration ------ */
+ 
+ /* This is used to query the native os for a list of font names and
+  * corresponding paths. The general idea is to save the hassle of
+  * building a custom fontmap file.
+  */
+ 
+void *gp_enumerate_fonts_init(gs_memory_t *mem)
+{
+    return NULL;
+}
+         
+int gp_enumerate_fonts_next(void *enum_state, char **fontname, char **path)
+{
+    return 0;
+}
+                         
+void gp_enumerate_fonts_free(void *enum_state)
+{
+}           
