@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gsdevice.c,v 1.24 2004/11/19 01:44:52 raph Exp $ */
+/* $Id: gsdevice.c,v 1.25 2005/10/12 17:59:55 leonardo Exp $ */
 /* Device operators for Ghostscript library */
 #include "ctype_.h"
 #include "memory_.h"		/* for memchr, memcpy */
@@ -457,6 +457,13 @@ gs_make_null_device(gx_device_null *dev_null, gx_device *dev,
     }
 }
 
+/* Is a null device ? */
+bool gs_is_null_device(gx_device *dev)
+{
+    /* Assuming null_fill_path isn't used elswhere. */
+    return dev->procs.fill_path == gs_null_device.procs.fill_path;
+}
+
 /* Mark a device as retained or not retained. */
 void
 gx_device_retain(gx_device *dev, bool retained)
