@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gsdparam.c,v 1.17 2004/06/29 21:01:41 dan Exp $ */
+/* $Id: gsdparam.c,v 1.18 2005/06/20 08:59:23 igor Exp $ */
 /* Default device parameters for Ghostscript library */
 #include "memory_.h"		/* for memcpy */
 #include "string_.h"		/* for strlen */
@@ -169,7 +169,7 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
 
     if (colors > 1) {
 	int RGBValues = dev->color_info.max_color + 1;
-	long ColorValues = (depth >= (8 * sizeof(gx_color_index)) ? -1
+	long ColorValues = (depth >= (8 * arch_sizeof_color_index) ? -1
 							: 1L << depth);
 
 	if ((code = param_write_int(plist, "RedValues", &RGBValues)) < 0 ||
