@@ -14,16 +14,28 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gsexit.h,v 1.4.2.1 2002/02/22 19:45:57 ray Exp $ */
+/* $Id: gsexit.h,v 1.10 2004/08/04 19:36:12 stefan Exp $ */
 /* Declarations for exits */
 
 #ifndef gsexit_INCLUDED
 #  define gsexit_INCLUDED
 
-/* These do not call exit() */
-/* You still need to return to the caller.*/
-void gs_exit_with_code(P2(int exit_status, int code));
-void gs_exit(P1(int exit_status));
-void gs_abort(P0());
+
+/** The client must provide this.
+ *  normally they do exit cleanup and error messaging
+ *  without calling system exit() returning to the caller.
+ */
+int gs_to_exit(const gs_memory_t *mem, int exit_status);
+
+/** some clients prefer this to return the postscript error code
+ * to the caller otherwise the same as gs_to_exit()
+ */
+int gs_to_exit_with_code(const gs_memory_t *mem, int exit_status, int code);
+
+/** The client must provide this.  
+ * After possible cleanup it may call gp_do_exit() which calls exit() in a platform
+ * independent way.  This is a fatal error so returning is not a good idea.
+ */
+void gs_abort(const gs_memory_t *mem);
 
 #endif /* gsexit_INCLUDED */
