@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gsmemory.c,v 1.4.2.1 2002/02/22 19:45:58 ray Exp $ */
+/* $Id: gsmemory.c,v 1.9 2004/08/04 19:36:12 stefan Exp $ */
 /* Generic allocator support */
 #include "memory_.h"
 #include "gdebug.h"
@@ -115,9 +115,14 @@ gs_resize_struct_array(gs_memory_t *mem, void *obj, uint num_elements,
     return gs_resize_object(mem, obj, num_elements, cname);
 }
 
-/* Allocate a structure using a "raw memory" allocator. */
+
+/* Allocate a structure using a "raw memory" allocator.
+ * really just an alias for gs_alloc_struct_immovable 
+ * with the clients false expectation that it is saving memory   
+ */
+ 
 void *
-gs_raw_alloc_struct_immovable(gs_raw_memory_t * rmem,
+gs_raw_alloc_struct_immovable(gs_memory_t * rmem,
 			      gs_memory_type_ptr_t pstype,
 			      client_name_t cname)
 {
@@ -290,10 +295,27 @@ ENUM_PTRS_BEGIN_PROC(basic_enum_ptrs)
 {
     const gc_struct_data_t *psd = pstype->proc_data;
 
+    /* This check is primarily for misuse of the alloc_struct_array */
+    /* with number of elements 0 and allocation not passing 'element' */
+    if (size == 0) {
+#ifdef DEBUG
+	dprintf2("  basic_enum_ptrs: Attempt to enum 0 size structure at 0x%lx, type: %s\n",
+		vptr, pstype->sname);
+#endif
+	return 0;
+    }
     if (index < psd->num_ptrs) {
 	const gc_ptr_element_t *ppe = &psd->ptrs[index];
 	EV_CONST char *pptr = (EV_CONST char *)vptr + ppe->offset;
 
+#ifdef DEBUG
+	/* some extra checking to make sure we aren't out of bounds */
+	if (ppe->offset > size - sizeof(void *)) {
+	    dprintf4("  basic_enum_ptrs: Attempt to enum ptr with offset=%d beyond size=%d: structure at 0x%lx, type: %s\n",
+		    ppe->offset, size, vptr, pstype->sname);
+	    return 0;
+	}
+#endif
 	switch ((gc_ptr_type_index_t)ppe->type) {
 	    case GC_ELT_OBJ:
 		return ENUM_OBJ(*(const void *EV_CONST *)pptr);
