@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gspath.h,v 1.2.6.1 2002/02/22 19:45:58 ray Exp $ */
+/* $Id: gspath.h,v 1.7 2004/05/11 10:41:55 igor Exp $ */
 /* Graphics state path procedures */
 /* Requires gsstate.h */
 
@@ -24,25 +24,24 @@
 #include "gspenum.h"
 
 /* Path constructors */
-int gs_newpath(P1(gs_state *)),
-    gs_moveto(P3(gs_state *, floatp, floatp)),
-    gs_rmoveto(P3(gs_state *, floatp, floatp)),
-    gs_lineto(P3(gs_state *, floatp, floatp)),
-    gs_rlineto(P3(gs_state *, floatp, floatp)),
-    gs_arc(P6(gs_state *, floatp, floatp, floatp, floatp, floatp)),
-    gs_arcn(P6(gs_state *, floatp, floatp, floatp, floatp, floatp)),
+int gs_newpath(gs_state *),
+    gs_moveto(gs_state *, floatp, floatp),
+    gs_rmoveto(gs_state *, floatp, floatp),
+    gs_lineto(gs_state *, floatp, floatp),
+    gs_rlineto(gs_state *, floatp, floatp),
+    gs_arc(gs_state *, floatp, floatp, floatp, floatp, floatp),
+    gs_arcn(gs_state *, floatp, floatp, floatp, floatp, floatp),
     /*
      * Because of an obscure bug in the IBM RS/6000 compiler, one (but not
      * both) bool argument(s) for gs_arc_add must come before the floatp
      * arguments.
      */
-    gs_arc_add(P8(gs_state *, bool, floatp, floatp, floatp, floatp, floatp, bool)),
-    gs_arcto(P7(gs_state *, floatp, floatp, floatp, floatp, floatp, float[4])),
-    gs_curveto(P7(gs_state *, floatp, floatp, floatp, floatp, floatp, floatp)),
-    gs_rcurveto(P7(gs_state *, floatp, floatp, floatp, floatp, floatp, floatp)),
-    gs_closepath(P1(gs_state *));
+    gs_arc_add(gs_state *, bool, floatp, floatp, floatp, floatp, floatp, bool),
+    gs_arcto(gs_state *, floatp, floatp, floatp, floatp, floatp, float[4]),
+    gs_curveto(gs_state *, floatp, floatp, floatp, floatp, floatp, floatp),
+    gs_rcurveto(gs_state *, floatp, floatp, floatp, floatp, floatp, floatp),
+    gs_closepath(gs_state *);
 
-/* Imager-level procedures */
 #ifndef gs_imager_state_DEFINED
 #  define gs_imager_state_DEFINED
 typedef struct gs_imager_state_s gs_imager_state;
@@ -51,25 +50,29 @@ typedef struct gs_imager_state_s gs_imager_state;
 #  define gx_path_DEFINED
 typedef struct gx_path_s gx_path;
 #endif
-int gs_imager_arc_add(P9(gx_path * ppath, gs_imager_state * pis,
-			 bool clockwise, floatp axc, floatp ayc,
-			 floatp arad, floatp aang1, floatp aang2,
-			 bool add_line));
+#ifndef gs_matrix_fixed_DEFINED
+#define gs_matrix_fixed_DEFINED
+typedef struct gs_matrix_fixed_s gs_matrix_fixed;
+#endif
 
-#define gs_arc_add_inline(pgs, cw, axc, ayc, arad, aa1, aa2, add)\
-  gs_imager_arc_add((pgs)->path, (gs_imager_state *)(pgs),\
-		    cw, axc, ayc, arad, aa1, aa2, add)
+/* Imager-level procedures */
+int gs_imager_arc_add(gx_path * ppath, gs_imager_state * pis,
+		      bool clockwise, floatp axc, floatp ayc,
+		      floatp arad, floatp aang1, floatp aang2,
+		      bool add_line);
+void make_quadrant_arc(gs_point *p, const gs_point *c, 
+	const gs_point *p0, const gs_point *p1, double r);
 
 /* Add the current path to the path in the previous graphics state. */
-int gs_upmergepath(P1(gs_state *));
+int gs_upmergepath(gs_state *);
 
 /* Path accessors and transformers */
-int gs_currentpoint(P2(gs_state *, gs_point *)),
-      gs_upathbbox(P3(gs_state *, gs_rect *, bool)),
-      gs_dashpath(P1(gs_state *)),
-      gs_flattenpath(P1(gs_state *)),
-      gs_reversepath(P1(gs_state *)),
-      gs_strokepath(P1(gs_state *));
+int gs_currentpoint(gs_state *, gs_point *),
+      gs_upathbbox(gs_state *, gs_rect *, bool),
+      gs_dashpath(gs_state *),
+      gs_flattenpath(gs_state *),
+      gs_reversepath(gs_state *),
+      gs_strokepath(gs_state *);
 
 /* The extra argument for gs_upathbbox controls whether to include */
 /* a trailing moveto in the bounding box. */
@@ -79,19 +82,18 @@ int gs_currentpoint(P2(gs_state *, gs_point *)),
 /* Path enumeration */
 
 /* This interface conditionally makes a copy of the path. */
-gs_path_enum *
-             gs_path_enum_alloc(P2(gs_memory_t *, client_name_t));
-int gs_path_enum_copy_init(P3(gs_path_enum *, const gs_state *, bool));
+gs_path_enum *gs_path_enum_alloc(gs_memory_t *, client_name_t);
+int gs_path_enum_copy_init(gs_path_enum *, const gs_state *, bool);
 
 #define gs_path_enum_init(penum, pgs)\
   gs_path_enum_copy_init(penum, pgs, true)
-int gs_path_enum_next(P2(gs_path_enum *, gs_point[3]));  /* 0 when done */
-void gs_path_enum_cleanup(P1(gs_path_enum *));
+int gs_path_enum_next(gs_path_enum *, gs_point[3]);  /* 0 when done */
+void gs_path_enum_cleanup(gs_path_enum *);
 
 /* Clipping */
-int gs_clippath(P1(gs_state *)),
-    gs_initclip(P1(gs_state *)),
-    gs_clip(P1(gs_state *)),
-    gs_eoclip(P1(gs_state *));
+int gs_clippath(gs_state *),
+    gs_initclip(gs_state *),
+    gs_clip(gs_state *),
+    gs_eoclip(gs_state *);
 
 #endif /* gspath_INCLUDED */
