@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gsptype2.c,v 1.17 2005/02/08 16:52:41 igor Exp $ */
+/* $Id: gsptype2.c,v 1.19 2005/05/25 15:57:58 igor Exp $ */
 /* PatternType 2 implementation */
 #include "gx.h"
 #include "gserrors.h"
@@ -315,3 +315,15 @@ gx_dc_pattern2_can_overlap(const gx_device_color *pdevc)
     }
 }
 
+/* Check whether a pattern color has a background. */
+bool gx_dc_pattern2_has_background(const gx_device_color *pdevc)
+{
+    gs_pattern2_instance_t * pinst;
+    const gs_shading_t *Shading;
+
+    if (pdevc->type != &gx_dc_pattern2)
+	return false;
+    pinst = (gs_pattern2_instance_t *)pdevc->ccolor.pattern;
+    Shading = pinst->template.Shading;
+    return !pinst->shfill && Shading->params.Background != NULL;
+}
