@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gxchar.c,v 1.44 2005/01/10 11:32:06 igor Exp $ */
+/* $Id: gxchar.c,v 1.47 2005/07/21 09:53:42 igor Exp $ */
 /* Default implementation of text writing */
 #include "gx.h"
 #include "memory_.h"
@@ -581,8 +581,33 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
 				 log2_scale.x + log2_scale.y > alpha_bits ?
 				 penum->dev_cache2 : NULL),
 				iwidth, iheight, &log2_scale, depth);
-	if (cc == 0)
-	    return 0;		/* too big for cache */
+	if (cc == 0) {
+	    /* too big for cache or no cache */
+	    gx_path box_path;
+
+	    if (penum->current_font->FontType != ft_user_defined && 
+		penum->current_font->FontType != ft_CID_user_defined) {
+		/* Most fonts don't paint outside bbox,
+		   so render with no clipping. */
+		return 0;
+	    }
+	    /* Render with a clip. */
+	    /* show_proceed already did gsave. */
+	    pgs->in_cachedevice = CACHE_DEVICE_NONE; /* Provide a correct grestore on error. */
+	    clip_box.p.x = penum->origin.x - fixed_ceiling(-cll.x);
+	    clip_box.p.y = penum->origin.y - fixed_ceiling(-cll.y);
+	    clip_box.q.x = clip_box.p.x + int2fixed(iwidth);
+	    clip_box.q.y = clip_box.p.y + int2fixed(iheight);
+	    gx_path_init_local(&box_path, pgs->memory);
+	    code = gx_path_add_rectangle(&box_path, clip_box.p.x, clip_box.p.y,
+						    clip_box.q.x, clip_box.q.y);
+	    if (code < 0)
+		return code;
+	    code = gx_cpath_clip(pgs, pgs->clip_path, &box_path, gx_rule_winding_number);
+	    gx_path_free(&box_path, "set_cache_device");
+	    pgs->in_cachedevice = CACHE_DEVICE_NONE_AND_CLIP;
+	    return 0;		
+	}
 	/* The mins handle transposed coordinate systems.... */
 	/* Truncate the offsets to avoid artifacts later. */
 	cc->offset.x = fixed_ceiling(-cll.x);
@@ -946,10 +971,6 @@ show_proceed(gs_show_enum * penum)
 			glyph = (*penum->encode_char)(pfont, chr,
 						      GLYPH_SPACE_NAME);
 			SET_CURRENT_GLYPH(penum, glyph);
-			if (glyph == gs_no_glyph) {
-			    cc = 0;
-			    goto no_cache;
-			}
 		    } else
     			SET_CURRENT_GLYPH(penum, glyph);
 		    penum->is_pure_color = gs_color_writes_pure(penum->pgs); /* Save
@@ -973,6 +994,10 @@ show_proceed(gs_show_enum * penum)
 				return code;
 			}
 			penum->pair = pair;
+			if (glyph == gs_no_glyph) {
+			    cc = 0;
+			    goto no_cache;
+			}
 			cc = gx_lookup_cached_char(pfont, pair, glyph, wmode,
 						   depth, &subpix_origin);
 		    }
@@ -1477,7 +1502,7 @@ private int
 show_cache_setup(gs_show_enum * penum)
 {
     gs_state *pgs = penum->pgs;
-    gs_memory_t *mem = pgs->memory;
+    gs_memory_t *mem = penum->memory;
     gx_device_memory *dev =
 	gs_alloc_struct(mem, gx_device_memory, &st_device_memory,
 			"show_cache_setup(dev_cache)");
