@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: stdio_.h,v 1.4.2.1 2002/02/22 19:46:00 ray Exp $ */
+/* $Id: stdio_.h,v 1.10 2003/12/09 04:04:08 giles Exp $ */
 /* Generic substitute for stdio.h */
 
 #ifndef stdio__INCLUDED
@@ -42,7 +42,7 @@
  * if they do, the declaration will be compatible with this one, as long
  * as const has not been disabled by defining it to be the empty string.
  */
-int unlink(P1(const char *));
+int unlink(const char *);
 #endif
 
 #endif
@@ -68,4 +68,9 @@ int unlink(P1(const char *));
 #  define SEEK_END 2
 #endif
 
+#if defined(_MSC_VER)
+#  define fdopen(handle,mode) _fdopen(handle,mode)
+#  define fileno(file) _fileno(file)
+#endif
+
 #endif /* stdio__INCLUDED */
