@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: zpacked.c,v 1.2.6.1 2002/02/22 19:46:01 ray Exp $ */
+/* $Id: zpacked.c,v 1.7 2004/08/19 19:33:09 stefan Exp $ */
 /* Packed array operators */
 #include "ghost.h"
 #include "ialloc.h"
@@ -26,6 +26,7 @@
 #include "ivmspace.h"
 #include "oper.h"
 #include "store.h"
+#include "gxalloc.h"
 
 /* - currentpacking <bool> */
 private int
@@ -104,7 +105,7 @@ make_packed_array(ref * parr, ref_stack_t * pstack, uint size,
 	pref = ref_stack_index(pstack, i - 1);
 	switch (r_btype(pref)) {	/* not r_type, opers are special */
 	    case t_name:
-		if (name_index(pref) >= packed_name_max_index)
+	      if (name_index(imem, pref) >= packed_name_max_index)
 		    break;	/* can't pack */
 		idest++;
 		continue;
@@ -150,7 +151,7 @@ make_packed_array(ref * parr, ref_stack_t * pstack, uint size,
 	}
 	ishort = idest += packed_per_ref;
     }
-    pad = -idest & (packed_per_ref - 1);	/* padding at end */
+    pad = -(int)idest & (packed_per_ref - 1);	/* padding at end */
 
     /* Now we can allocate the array. */
 
@@ -173,7 +174,7 @@ make_packed_array(ref * parr, ref_stack_t * pstack, uint size,
 	switch (r_btype(pref)) {	/* not r_type, opers are special */
 	    case t_name:
 		{
-		    uint nidx = name_index(pref);
+		    uint nidx = name_index(imem, pref);
 
 		    if (nidx >= packed_name_max_index)
 			break;	/* can't pack */
@@ -220,7 +221,7 @@ make_packed_array(ref * parr, ref_stack_t * pstack, uint size,
 	    while (--i >= 0) {
 		--psrc;
 		--pmove;
-		packed_get(psrc, pmove);
+		packed_get(imem->non_gc_memory, psrc, pmove);
 	    }
 	}
 	pshort = pdest += packed_per_ref;
